/*
 *  NAME:
 *  	spica_read_1a.c
 *
 *  PURPOSE:
 *	This function reads the SPICAM and SPICAV UV/IR data in level 1A.
 *      The function returns the C typedef (spica_1Afile) corresponding to the 
 *      FITS file. To get more informations about the content of the returned 
 *      data type, please read the PDF file named "Using-SPICAM-V-1A-data.pdf" 
 *      or the CONTENT_UV.TXT/CONTENT_IR.TXT
 *
 * Inputs:
 *      spica_read_1A:
 *      - spica_1Afile  [*spica_1Afile]  : spica_1Afile typedef pointer
 *      - spica_1Afname [*char]          : full filename pointer
 *      - mode          [Int]            : output messages mode, 
 *                                         0:output messages disabled; 
 *                                         1: output messages enabled
 *      - no_mask       [Int]            : 0:The flag mask will be appliked,
 *                                         1:The flag mask will NOT be applied
 *				           This parameter should be only defined
 *                                         for UV data
 *
 *
 * Library:
 *      cfitsio: http://heasarc.gsfc.nasa.gov/fitsio/
 *      To compile this module: - Import the static library "libcfitsio.a" in 
 *                                your system library path
 *                              - Add the option "-I cfitsio" or 
 *                               "out libcfitsio.a" in you compiler command line
 *
 * Author:
 *   Nicolas Chapron
 *     CNRS | LATMOS - Laboratoire Atmospheres, Milieux, Observations Spatiales.
 *     11 boulevard d'Alembert
 *     78280 Guyancourt
 *     France
 *     Tel. 33-(0) 1 80 28 50 57
 *     Email. nicolas.chapron@latmos.ipsl.fr
 *
 *  SVN Revision:
 *      $Rev: 2371 $
 *      $Author: chapron $
 *      $Date: 2014-11-03 10:50:53 +0100 (Lun, 03 nov 2014) $
 */

#include "spica_read_1A.h"

/****************************************************************************/
void 
spica_read_1A(spica_1Afile *output,char *spica_1Afname, int mode, int no_mask)
{
	
	fitsfile *fptr;
	int status = 0;
	char msg[32];
	char *str_uv;
	
	// Open FITS file
	fits_open_file(&fptr, spica_1Afname, READONLY, &status);
	
	if(status != 0){
		fits_get_errstatus(status,msg);
		printf("CFITSIO error %d:%s\n",status,msg);
		exit(0);
	}
	
	// Check filename
	str_uv = strstr(spica_1Afname,"1AU");
	
	if(str_uv != NULL){
		if(mode != 0)
			printf("1A UV reading process started...\n");
		spica_read_1AUV(fptr,output,mode,no_mask);	
	}
	else{
		if(mode != 0)
			printf("1A IR reading process started...\n");
		spica_read_1AIR(fptr,output,mode,no_mask);			
	}
	
	//Close FITS file
	fits_close_file(fptr, &status);
}
/****************************************************************************/

void
spica_read_1AUV(fitsfile *fptr, spica_1Afile *output, int mode, int no_mask)
{
	int uvir = 0;
	int status = 0;
	int check_info = 1;
	int check_param = 1;
	int check_geoinfo = 1;
	int check_clean = 1;
	int check_err = 1;
	int check_flag = 1;
	int check_geo = 1;
	int check_all = 1;
		
	read_Info(fptr,output,&status,uvir);
	if(status == 0){
		if(mode != 0)
			printf("Info successfully read\n");
		check_info = 0;
	}
	else
		printf("Error while reading Info\n");
	

	read_CleanData(fptr,output,&status);
	if(status == 0){
		if(mode != 0)
			printf("Clean data successfully read\n");
		check_clean = 0;
	}
	else
		printf("Error while reading Clean data\n");
	

	read_ErrData(fptr,output,&status);
	if(status == 0){
		if(mode != 0)
			printf("Error data successfully read\n");
		check_err = 0;
	}
	else
		printf("Error while reading Error data\n");
	
	read_Flag(fptr,output,&status);
	if(status == 0){
		if(mode != 0)
			printf("Flag data successfully read\n");
		check_flag = 0;
	}
	else
		printf("Error while reading Flag data\n");
	
    	if(no_mask == 0){
    		
		apply_Flag(output);
		if(mode != 0)
			printf("Flag mask applied\n");
	}
		
	read_Parameters(fptr,output,&status,0,0,uvir);
	if(status == 0){
		if(mode != 0)
			printf("Functional parameters successfully read\n");
		check_param = 0;
	}
	else
		printf("Error while reading Functional parameters\n");
	
	read_GeoInfo(fptr,output,&status,uvir);
	if(status == 0){
		if(mode != 0)
			printf("Geo information successfully read\n");
		check_geoinfo = 0;
	}
	else
		printf("Error while reading Geo information\n");
	
	read_Geo(fptr,output,&status,mode,uvir);
	if(status == 0){
		if(mode != 0)
			printf("Geo data successfully read\n");
		check_geo = 0;
	}
	else
		printf("Error while reading Geo data\n");
	
	check_all = check_info + check_param + check_geoinfo + check_clean + check_geo + check_err;
	if(check_all == 0 && mode != 0) 
		printf("The file was read succesfully!\n");

}
/****************************************************************************/

void
spica_read_1AIR(fitsfile *fptr, spica_1Afile *output, int mode, int no_mask)
{
	int uvir = 1;
	int status = 0;
	int check_info = 1;
	int check_param = 1;
	int check_geoinfo = 1;
	int check_clean = 1;
	int check_wl = 1;
	int check_dc = 1;
	int check_raw = 1;
	int check_geo = 1;
	int check_tor = 1;
	int check_all = 1;

	read_Info(fptr,output,&status,uvir);
	if(status == 0){
		if(mode != 0)
			printf("Info successfully read\n");
		check_info = 0;
	}
	else
		printf("Error while reading Info\n");

	read_CleanData(fptr,output,&status);
	if(status == 0){
		if(mode != 0)
			printf("Clean data successfully read\n");
		check_clean = 0;
	}
	else
		printf("Error while reading Clean data\n");

	read_WL(fptr,output,&status);
	if(status == 0){
		if(mode != 0)
			printf("WAVELENGTH data successfully read\n");
		check_wl = 0;
	}
	else
		printf("Error while reading WAVELENGTH data\n");	
	
	read_DC(fptr,output,&status);
	if(status == 0){
		if(mode != 0)
			printf("DC data successfully read\n");
		check_dc = 0;
	}
	else
		printf("Error while reading DC data\n");	

	read_RAW(fptr,output,&status);
	if(status == 0){
		if(mode != 0)
			printf("RAW data successfully read\n");
		check_raw = 0;
	}
	else
		printf("Error while reading RAW data\n");	

	read_Time_of_Records(fptr,output,&status);
	if(status == 0){
		if(mode != 0)
			printf("Time_of_Records data successfully read\n");
		check_tor = 0;
	}
	else
		printf("Error while reading Time_of_Records data\n");	
	
	read_Parameters(fptr,output,&status,0,0,uvir);
	if(status == 0){
		if(mode != 0)
			printf("Functional parameters successfully read\n");
		check_param = 0;
	}
	else
		printf("Error while reading Functional parameters\n");
	
	read_GeoInfo(fptr,output,&status,uvir);
	if(status == 0){
		if(mode != 0)
			printf("Geo information successfully read\n");
		check_geoinfo = 0;
	}
	else
		printf("Error while reading Geo information\n");
	
	read_Geo(fptr,output,&status,mode,uvir);
	if(status == 0){
		if(mode != 0)
			printf("Geo data successfully read\n");
		check_geo = 0;
	}
	else
		printf("Error while reading Geo data\n");
	
	check_all = check_info + check_param + check_geoinfo + check_geo + check_clean + check_dc + check_raw + check_wl;
	
	if(check_all == 0 && mode != 0) 
		printf("The file was read succesfully!\n");	
	

	
}
/****************************************************************************/

